# # -*- coding: utf-8 -*- #
# frozen_string_literal: true

# describe Rouge::Lexers::Sml do
#   let(:subject) { Rouge::Lexers::Sml.new }
# 
#   describe 'guessing' do
#     include Support::Guessing
# 
#     it 'guesses by filename' do
#       assert_guess :filename => 'foo.???'
#     end
# 
#     it 'guesses by mimetype' do
#       assert_guess :mimetype => 'text/x-sml'
#       assert_guess :mimetype => 'application/x-sml'
#     end
# 
#     it 'guesses by source' do
#       assert_guess :source => '????'
#     end
#   end
# end
# 
